/* connect.c
   Module for handling of generic network connections */

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/nameser.h>
#include <netinet/in.h>
#include <errno.h>
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */

#include "geturl.h"
#include "connect.h"

/* Global variables for sharing between bindport
   and acceptport: */
int msock, addrlen;
struct sockaddr *addr;
struct sockaddr_in srv;


/* Thanks to Miroslav Zagorac <zaga@fly.cc.fer.hr> */
uerr_t
make_connection(int *sock, char *hostname, int port)
{
   struct sockaddr_in sock_name;
   struct hostent *hptr;

   /* Get internet address of the host */
   if (!(hptr = gethostbyname(hostname)))
      return HOSTERR;

   /* Set port and protocol */
   sock_name.sin_family = AF_INET;
   sock_name.sin_port = htons(port);
   memcpy(&sock_name.sin_addr, hptr->h_addr, hptr->h_length);
   /* Make internet socket */
   if ((*sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return CONSOCKERR;
   
   /* Connect to remote host */
   if (connect(*sock, (struct sockaddr *) &sock_name, sizeof (sock_name)))
   {
      if (errno == ETIMEDOUT)
	 return CONTIMEDOUT;
      else if (errno == ECONNREFUSED)
	 return CONREFUSED;
      else
	 return CONERROR;
   }
   return NOCONERROR;
}

/* Thanks to Zlatko Calusic <maverick@fly.cc.fer.hr> */

/* bindport does all the work necessary to bind a port, creating a
   master socket msock. Use acceptport to call accept and receive the
   actual socket. */
uerr_t
bindport(int *port)
{
   int optval = 1;

   addr = (struct sockaddr *) & srv;
   addrlen = sizeof(srv);
   if ((msock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      return CONSOCKERR;
   if (setsockopt(msock, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, sizeof(optval)) < 0)
      return CONSOCKERR;
   srv.sin_family = AF_INET;
   srv.sin_addr.s_addr = htonl(INADDR_ANY);
   srv.sin_port = htons(*port);
   if (bind(msock, addr, addrlen) < 0)
      return BINDERR;
   if (!*port)
   {
      if (getsockname(msock, addr, &addrlen) < 0)
	 return CONPORTERR;
      *port = ntohs(srv.sin_port);
   }
   if (listen(msock, 1) < 0)
      return LISTENERR;
   return BINDOK;
}

/* Acceptport uses msock and addrlen to call the accept system call
   and return the data socket of the connection. It blocks the caller,
   as per accept. */
uerr_t
acceptport(int *sock)
{
   if ((*sock = accept(msock, addr, &addrlen)) < 0)
      return ACCEPTERR;
   return ACCEPTOK;
}

/* Calls shutdown and closes the sockets. */
void
closeport(int sock)
{
   shutdown(sock, 2);
   close(sock);
   close(msock);
}
